/*
 * Copyright 2010, Intel Corporation
 *
 * This file is part of PowerTOP
 *
 * This program file is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program in a file named COPYING; if not, write to the
 * Free Software Foundation, Inc,
 * 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301 USA
 * or just google for it.
 *
 * Authors:
 *	Srinivas Pandruvada<Srinivas.Pandruvada@linux.intel.com>
 */
#include <cstdlib>
#include <cstdio>
#include "../parameters/parameters.h"
#include "../lib.h"
#include "gpu_rapl_device.h"

gpu_rapl_device::gpu_rapl_device(i915gpu *parent)
	: i915gpu(),
	  device_valid(false)
{
	last_time = pt_gettime();
	if (rapl.pp1_domain_present()) {
		device_valid = true;
		parent->add_child(this);
		rapl.get_pp1_energy_status(&last_energy);
	}
}

void gpu_rapl_device::start_measurement(void)
{
	if (!device_valid)
		return;
	last_time = pt_gettime();
	rapl.get_pp1_energy_status(&last_energy);
}

void gpu_rapl_device::end_measurement(void)
{
	if (!device_valid)
		return;
	const struct timeval	curr_time = pt_gettime();
	const double		delta = (curr_time.tv_sec - last_time.tv_sec)
			      + (curr_time.tv_usec - last_time.tv_usec) / 1000000.0;
	double energy;

	consumed_power = 0.0;
	if (delta >= 0.00001) {
		rapl.get_pp1_energy_status(&energy);
		consumed_power = (energy - last_energy) / delta;
		last_energy = energy;
		last_time = curr_time;
	}
}

double gpu_rapl_device::power_usage([[maybe_unused]] struct result_bundle *result, [[maybe_unused]] struct parameter_bundle *bundle)
{
	if (rapl.pp1_domain_present())
		return consumed_power;
	else
		return 0.0;
}

void gpu_rapl_device::collect_json_fields(std::string &_js)
{
    i915gpu::collect_json_fields(_js);
    JSON_KV("last_time_sec",  (long)last_time.tv_sec);
    JSON_KV("last_time_usec", (long)last_time.tv_usec);
    JSON_FIELD(last_energy);
    JSON_FIELD(consumed_power);
    JSON_FIELD(device_valid);
}
