/* Copyright (c) 2012 Samsung Electronics Co., Ltd.
 *	http://www.samsung.com/
 *
 * This file is part of PowerTOP
 *
 * This program file is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program in a file named COPYING; if not, write to the
 * Free Software Foundation, Inc,
 * 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301 USA
 * or just google for it.
 *
 * Common part of report_formatter_csv and report_formatter_html.
 * Written by Igor Zhbanov <i.zhbanov@samsung.com>
 * 2012.10 */

#pragma once

#include "report-formatter.h"

/*
 * # subclasses of report_formatter_string_base
 *
 * | subclass               | filename                                    |
 * | ---------------------- | ------------------------------------------- |
 * | report_formatter_csv   | src/report/report-formatter-csv.h           |
 * | report_formatter_html  | src/report/report-formatter-html.h          |
 * | report_formatter_md    | src/report/report-formatter-md.h            |
 */
class report_formatter_string_base: public report_formatter
{
public:
	virtual std::string get_result() const override;
	virtual void clear_result() override;

	virtual void add(const std::string &str) override;

protected:
	void add_exact(const std::string &str);

	virtual std::string escape_string(const std::string &str) = 0;

	std::string result;
};

