/*
 * Copyright 2010, Intel Corporation
 *
 * This file is part of PowerTOP
 *
 * This program file is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program in a file named COPYING; if not, write to the
 * Free Software Foundation, Inc,
 * 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301 USA
 * or just google for it.
 *
 * Authors:
 *	Arjan van de Ven <arjan@linux.intel.com>
 */

#include <memory>

#include "tuning.h"
#include "tunable.h"
#include "../lib.h"

std::vector<std::unique_ptr<tunable>> all_tunables;
std::vector<std::unique_ptr<tunable>> all_untunables;


tunable::tunable(const std::string &str, double _score, [[maybe_unused]] const std::string &good, [[maybe_unused]] const std::string &bad, [[maybe_unused]] const std::string &neutral)
{
	desc = str;
	score = _score;
}


tunable::tunable(void)
{
	score = 0;
	desc = "";
	// All fields are already defaulted in header
}

void tunable::collect_json_fields(std::string &_js)
{
	JSON_FIELD(desc);
	JSON_FIELD(score);
	JSON_KV("result", result_string());
	JSON_FIELD(toggle_good);
	JSON_FIELD(toggle_bad);
}
